#include "NFC.h"
#include <SPI.h>
#include <PubSubClient.h>
#include <DESFire.h>
#include "helpers.h"

NFC::NFC(int pin_ss, int pin_rst)
{
    SPI.begin();

    rfid = new DESFire(pin_ss, pin_rst);
    rfid->PCD_Init();
}

bool NFC::hasNewCard()
{
    if(rfid->PICC_IsNewCardPresent() && rfid->PICC_ReadCardSerial() && rfid->uid.sak == 0x20)
    {
        Serial.println("New Card detected");
        return true;
    }
    return false;
}

MFRC522::Uid NFC::getUID()
{
    return uid;
}

bool NFC::selectCard()
{
    MFRC522::StatusCode status;
    byte ats_buffer[16];
    byte ats_buffer_len = 16;

    status = rfid->PICC_RequestATS(ats_buffer, &ats_buffer_len);
    if (status != MFRC522::STATUS_OK)
    {
        Serial.printf("PICC_RequestATS: 0x%02x\n", status);
        return false;
    }

    status = rfid->PICC_ProtocolAndParameterSelection(0x00, 0x11, 0x00);
    if (status != MFRC522::STATUS_OK)
    {
        Serial.printf("PICC_ProtocolAndParameterSelection: 0x%02x\n", status);
        return false;
    }

    cardSelected = true;
    uid = rfid->uid;
    Serial.println("Card Selected");
    return true;
}

bool NFC::deselectCard()
{
    cardSelected = false;
    Serial.println("Card Deselected");

    return true;
}

bool NFC::hasCardSelected()
{
    return cardSelected;
}

bool NFC::isCardLost()
{
    uint8_t control = 0x00;
    for(int i=0; i<3; i++)
    {
        if(!rfid->PICC_IsNewCardPresent())
        {
            if(rfid->PICC_ReadCardSerial())
            {
                control |= 0x16;
            }
            if(rfid->PICC_ReadCardSerial())
            {
                control |= 0x16;
            }
            control += 0x1;
        }
        control += 0x4;
    }

    if(!(control == 13 || control == 14))
    {
        Serial.println("Card Lost");
        return true;
    }
    else
    {
        return false;
    }
}

MFRC522::StatusCode NFC::Transceive(byte* command, byte command_len, byte* response, byte* response_len)
{
    byte buffer[APDU_BUFFER_SIZE + 4];
    byte buffer_len = command_len + 4;
    MFRC522::StatusCode status;

    buffer[0] = 0x0A;
	buffer[1] = 0x00;
    memcpy(&buffer[2], command, command_len);

    status = rfid->PCD_CalculateCRC(buffer, buffer_len - 2, &buffer[buffer_len - 2]);
	if (status != MFRC522::STATUS_OK)
    {
        Serial.printf("PCD_CalculateCRC: 0x%02x\n", status);
		return status;
	}

    // Serial.println(buffer_len);
    // char command_h[APDU_BUFFER_SIZE*2] = { 0 };
    // bytes2chars(buffer, buffer_len, command_h, true);
    // Serial.println(command_h);
    
    status = rfid->PCD_TransceiveData(buffer, buffer_len, response, response_len);
	if (status != MFRC522::STATUS_OK)
    {
        Serial.printf("PCD_TransceiveData: 0x%02x\n", status);
		return status;
	}

    return status;
}